[![CircleCI](https://circleci.com/gh/IDAES/ddm-learning.svg?style=svg&circle-token=7e4976a371153c4c7256bafa7aa4793be3676a25)](https://circleci.com/gh/IDAES/ddm-learning)

# Thermodynamics-and-kinetics

This repository contains ALAMO and RIPE wrappers for the
IDAES project.

## Contributing

**By contributing to this repository, you are agreeing to all the terms set out
in the LICENSE.txt and COPYRIGHT.txt files in this directory.**

## ALAMOPy: Python interface to ALAMO

The ALAMO Python module facilitates easy access to ALAMO and provides additional tools for uncertainty analysis. Detailed documentation of ALAMO Python can be found in [ALAMOPy documentation](./documentation.pdf)

### Installation

Change to the `alamo_python` subdirectory and use the standard
Python install formula:

    python setup.py install


### Usage

The ALAMOpy wrapper can be used to call alamo from Python. ALAMO should be located in the user's path, or the location of the ALAMO executable can be provided to the wrapper as a key word arguement. 

ALAMO can be called through the wrapper using the doalamo subroutine

alamopy.doalamo(xdata,zdata)

### Results

By default, two files will be generated:
* logscratch - A file containing the output ALAMO provide to the terminal
* tempalm.py - A Python usable file named tempalm.py by default, this file contains the resultant ALAMO model. The key word argument almname can be used to control the name of this file.

The python file produced by ALAMOpy can be used by importing it into Python.

import tempalm.py
z = tempalm.f(x)

Where x is an numpy array with observations in the first index and input variables in the second. The results are returned in an n dimension numpy array.

#### Additional results and options

res = alamopy.doalamo(xdata,zdata)

If the results of the call are assigned to a varaible, additional information is provided in a dictionary including:
* res['model']    - a string representation of the resultant ALAMO model
* res['f(model)'] - a lambda function of the resultant model
* res['ssr']      - the sum of squared residuals on the provided training set

The ALAMO wrapper allows for any options defined in the ALAMO user manual to be appended directly to any constructed .alm file by using the keyword argument almopt='/optfile'.

The additional keyword arguemnts may be provided directly in the call to alamopy.doalamo including:
* All basis fucntion transformations availible through ALAMO (monomialpower, multi2power, etc... )
* xlabels - labels given to input variables as a list of strings
* zlabels - labels given to outputs
* xval - validaiton data for alamo
* zval - response validation data for alamo
* modeler - modeler value used in alamo
* almname - specify a name for the .alm file and resultant .py model file
* savescratch - saves .alm and .lst
* savetrace - saves trace file
* expandoutput - True/False that provides additional information in the output dictionary
* almpath - path to the alamo executable
* simulator - path to a simulator to interface with ALAMO

The additional output information provided by the expandoutput option is also contained in the associated ALAMO .trc and .lst files. A thorough explanation can be obtained in the ALAMO manual.


## ALAMOPy Examples

Four examples are included with the current version of alamopy:
* [linearex.py](./linearex.py)
* [sixhumpcamel.py](./sixhumpcamel.py) with a [Jupyter notebook](./sixhumpcameljup.ipynb)
* [lim.py](./lim.py)
* [branin.py](./branin.py)

## RIPE : Reaction Identification and Parameter Estimation

The RIPE module provides tools for reaction network identification. RIPE uses reactor data consisting of concentration, or conversion, values for multiple species that are obtained dynamically, or at multiple process conditions (temperatures, flow rates, working volumes) to identify probable reaction kinetics.

### Installation

Change to the `ripe_python` subdirectory and use the standard
Python install formula:

    python setup.py install

### Usage

RIPE is used by calling the module on an input file

ripe.model('input.ripe')

### Results

By default, one file will be generated
* output.txt - a file containing the selected reactions and parameter estimates

### Additional options

Options are provided to RIPE using a data file. Options that can be included in the .ripe file include:
* slabels - Species names
* pclabels - Process conditions over which data is indexed (T or c0)
* npc - Number of different process conditions over which data is obtained
* begin_c0data // end_c0data - Initial concentration data for each species with line breaks, space delimited
* begin_tdata // end_tdata - Temperature data for each process condition with line breaks
* begin_vdata // end_vdata - Working volume data for each process condition with line breaks
* timeseries - Number of sets of dyanmic data
* ntimedata - Number of datapoints for each timeseries
* begin_concdata // end_concdata - Process data with line breaks. Time series data should be entered with the time value in the first column

* mechanisms - reaction mechanisms to be considered:
    * iT - Isothermal rate without Arrhenious relationship
    * ma - Irreversible mass action kinetics
    * clc - Empirical models associated with chemical looping combustion reactions

After specifying clc in the list of possible mechanisms, an option 'clc' may be speicifed again along with a space delimited list c1-c19 (clc c1 c2...) to control the consideration of all possible clc reactions. The 19 empirical rate forms included relate specifically to catalyst conversion in chemical looping combustion reactors include:

* Random nucleation
* Power law models
* Avrami-Erofeev models

### RIPE Examples

One example is included with the current version of RIPE, and by be called by:

    python clc.py

All of these examples are built for Linux machines. They can be called from the command line by calling python directly, or can be called from inside a python environment using execfile().
