##############################################################################
# Institute for the Design of Advanced Energy Systems Process Systems
# Engineering Framework (IDAES PSE Framework) Copyright (c) 2018, by the
# software owners: The Regents of the University of California, through
# Lawrence Berkeley National Laboratory,  National Technology & Engineering
# Solutions of Sandia, LLC, Carnegie Mellon University, West Virginia
# University Research Corporation, et al. All rights reserved.
# 
# Please see the files COPYRIGHT.txt and LICENSE.txt for full copyright and
# license information, respectively. Both files are also available online
# at the URL "https://github.com/IDAES/idaes".
##############################################################################
def almpywriter(data, cvfun):
    # write '<almname>alm.py'
    # This funciton writes two files
    # - <almname>alm.py
    #    y=<fname>.f(X)
    # - <almname>cv.py
    #    y=<fname>.f(X,params)

    import re
    model = data['results']['model'].split('=')[1]
    model = model+' '
    tlist = ('sin','cos','log','exp')
    for tok in tlist:
        model = model.replace(tok,'np.'+tok)

    model = model.replace('^','**')
    # save coefmodel before changing model anymore
    coefmodel = re.sub("(?:.\w)?([0-9]+\.[0-9]+)",'b',model)
    coefmodel = re.sub("[E]{1}.\d{3}",'',coefmodel)

    with open(data['almstropts']['almname'].split('.')[0]+'alm.py', 'w') as r:
        r.write('def f(X):\n')
        r.write('    import numpy as np\n')
        r.write('    ndata=np.shape(X)[0]\n')
        r.write('    ninputs=np.shape(X)[1]\n')
        r.write('    for i in range(ninputs):\n')
        i = 0
        for lab in data['almlabs']['savexlabels']:
            r.write('        '+lab+'= X[:,'+str(i)+']\n')
            i=i+1
#        line = '    '
        for inp,tp in zip(data['almlabs']['savexlabels'],[',']*(len(data['almlabs']['savexlabels'])-1)+['']):
#            line=line+inp+tp
            model = model.replace('*('+inp+'*','*('+inp+'[i]*')
            model = model.replace('* ('+inp+'*','* ('+inp+'[i]*')
            model = model.replace('*'+inp,'*'+inp+'[i]')
            model = model.replace('('+inp+')','('+inp+'[i])')
            model = model.replace(' '+inp+' ',' '+inp+'[i] ')
            model = model.replace(' '+inp+'*',' '+inp+'[i]*')
#        line=line+'=np.ndarray.tolist(np.transpose(X))\n'
#        r.write(line)
#        r.write('    if(ninputs ==1):\n')
#        r.write('        '+data['results']['xlabels'][0]+'='+data['results']['xlabels'][0]+'[0]\n')
        r.write('    model=np.ones(ndata)\n')
        r.write('    for i in range(ndata):\n')
        r.write('        model[i]='+model+'\n')
        r.write('    return model\n')
        
    if cvfun:
        coefmodel = coefmodel.replace('-','+')
        tind = 0
        tstr = ''
        while 'b' in coefmodel:
            coefmodel = coefmodel.replace('b','B['+str(tind)+']',1)
            tstr=tstr+',B'+str(tind)
            tind=tind+1
        with open(data['almstropts']['almname'].split('.')[0]+'cv.py', 'w') as r:
            line = 'def f(X,params):\n'
            r.write(line)
            r.write('    import numpy as np\n')
            for inp,tp in zip(data['almlabs']['savexlabels'],[',']*(len(data['almlabs']['savexlabels'])-1)+['']):
                coefmodel = coefmodel.replace('*('+inp+'*','*('+inp+'[i]*')
                coefmodel = coefmodel.replace('* ('+inp+'*','* ('+inp+'[i]*')
                coefmodel = coefmodel.replace('*'+inp,'*'+inp+'[i]')
                coefmodel = coefmodel.replace('('+inp+')','('+inp+'[i])')
                coefmodel = coefmodel.replace(' '+inp+' ',' '+inp+'[i] ')
                coefmodel = coefmodel.replace(' '+inp+'*',' '+inp+'[i]*')
            r.write('    if (len(np.shape(X)) > 1):\n')
            r.write('        ndata,ninputs = np.shape(X)\n')
            r.write('    elif (len(params) > 1):\n')
            r.write('        ndata = 1\n')
            r.write('        ninputs = len(params)\n')
            r.write('    else:\n')
            r.write('        ndata = len(X)\n')
            r.write('        ninputs = 1\n')
            r.write('    for i in range(ninputs):\n')
            i = 0
            for lab in data['almlabs']['savexlabels']:
                r.write('        '+lab+'= X[:,'+str(i)+']\n')
                i=i+1
            r.write("    B = [0.0 for x in range("+data['results']['size']+")]\n")
            for i in range(int(data['results']['size'])):
                r.write('    B['+str(i)+'] = params[int('+str(i)+')]\n')
            r.write('    i=0\n')
            r.write('    model = np.ones(ndata)\n')
            r.write('    for i in range(ndata):\n')
            r.write('        model[i] = '+coefmodel+'\n')
            r.write('    model = model.astype(float)\n')
            r.write('    return model\n')
        
