##############################################################################
# Institute for the Design of Advanced Energy Systems Process Systems
# Engineering Framework (IDAES PSE Framework) Copyright (c) 2018, by the
# software owners: The Regents of the University of California, through
# Lawrence Berkeley National Laboratory,  National Technology & Engineering
# Solutions of Sandia, LLC, Carnegie Mellon University, West Virginia
# University Research Corporation, et al. All rights reserved.
# 
# Please see the files COPYRIGHT.txt and LICENSE.txt for full copyright and
# license information, respectively. Both files are also available online
# at the URL "https://github.com/IDAES/idaes".
##############################################################################
def almwriter(data, debug, vargs, kwargs):
    # This function writes a .alm file for the given data
    from alamopy.multos import deletefile, movefile, catfile

    xdata = vargs[0]
    zdata = vargs[1]
    if (debug['validation']):
        xvaldata = vargs[2]
        zvaldata = vargs[3]

    with open(data['almstropts']['almname'], 'w') as a:
        for arg in data['almopts'].keys():
            if arg == 'sigma' and data['almopts'][arg] < 0 : continue
            elif (arg == 'modeler' and data['almopts'][arg] == 6):
                a.write(arg+' '+str(data['almopts'][arg])+'\n')
                a.write('builder 0\n')
                a.write('enumall 1\n')
                a.write('enum1 1\n')          
            else: a.write(arg+' '+str(data['almopts'][arg])+'\n')

        for arg in list(['xlabels','zlabels']):
            a.write(arg+' '+str(data['almlstopts'][arg])[1:-1].replace(",",'').replace("'",'')+'\n')
        
        for arg in data['almlstopts'].keys():
            if arg in kwargs.keys() and arg not in list(['xlabels','zlabels']):
                temp=arg+' '
                for value in data['almlstopts'][arg]:
                    value=''.join( c for c in str(value) if c not in '(),')
                    temp=temp+str(value)+' '
                    temp=temp.replace('[','')
                    temp=temp.replace(']','')
                a.write(temp+'\n')

        a.write('trace 1\n')
        for arg in data['almstropts'].keys():
            if (arg not in list(['almopt','almname'])):
                a.write(arg+' '+str(data['almstropts'][arg])+'\n')

        # xmax and xmin writing in almset4
        if ('xmin' in kwargs.keys()):
            for arg in data['almset4'].keys():
                temp=arg+' '
                for value in data['almset4'][arg]:
                    value=''.join( c for c in str(value) if c not in "(),[] ")
                    temp=temp+str(value)+' '
                a.write(temp+'\n')
        else:
            for arg in data['almset4'].keys():
                a.write(arg+' '+str(data['almset4'][arg])[0:-1].replace(",",'')+'\n')

        # Write data and valdata
        a.write('begin_data\n')
        for i in range(data['almopts']['ndata']):
            temp = ''
            for j in range(data['almopts']['ninputs']):
                temp=temp+str(xdata[i][j])+' '
            for j in range(data['almopts']['noutputs']):
                temp=temp+str(zdata[i][j])+' '
            a.write(temp+'\n')
        a.write('end_data\n')
        if(debug['validation']):
            a.write('begin_valdata\n')
            for i in range(data['almopts']['nvaldata']):
                temp = ''
                for j in range(data['almopts']['ninputs']):
                    temp=temp+str(xvaldata[i][j])+' '
                for j in range(data['almopts']['noutputs']):
                    temp=temp+str(zvaldata[i][j])+' '
                a.write(temp+'\n')
            a.write('end_valdata\n')
        
        if data['almopts']['solvemip'] == 1:
            a.write('gams '+debug['gamsloc']+'\n')
        # make modeler=6 OLS
        a.write('cvxbic 1\n')
        a.write('convpen 0\n')

        # This option can be used to test the flow of alamo
        if (debug['hardset']):
            temp = 'maxterms '
            for i in range(data['almopts']['noutputs']):
                temp=temp+'3 '
            a.write(temp)

    # Append text file if specified
    if ('almopt' in data['almstropts'].keys()):
        catfile('almtemp',str(data['almstropts']['almname']),str(data['almstropts']['almopt']))
        deletefile(str(data['almstropts']['almname']))
        movefile('almtemp '+str(data['almstropts']['almname']))
