##############################################################################
# Institute for the Design of Advanced Energy Systems Process Systems
# Engineering Framework (IDAES PSE Framework) Copyright (c) 2018, by the
# software owners: The Regents of the University of California, through
# Lawrence Berkeley National Laboratory,  National Technology & Engineering
# Solutions of Sandia, LLC, Carnegie Mellon University, West Virginia
# University Research Corporation, et al. All rights reserved.
# 
# Please see the files COPYRIGHT.txt and LICENSE.txt for full copyright and
# license information, respectively. Both files are also available online
# at the URL "https://github.com/IDAES/idaes".
##############################################################################
import alamopy

# Import additional python modules for creating synthetic data
import math
import random
import numpy as np

#Specify number of training and validation points
ndata = 40
nvaldata=1000

random.seed(200)
xdatanp=np.zeros([ndata,2])
zdatanp=np.zeros([ndata,1])
xvaldatanp=np.zeros([nvaldata,2])
zvaldatanp=np.zeros([nvaldata,1])
inputs = list(['x1','x2'])
#Construct input dictionary
for i in range(ndata):
    xdatanp[i][0]=random.uniform(0,1)
    xdatanp[i][1]=random.uniform(0,1)
    zdatanp[i]=(1.0/6.0)*((30.0+5.0*xdatanp[i][0]*np.sin(5.0*xdatanp[i][0]))*(4.0+np.exp(-5.0*xdatanp[i][1]))-100.0)
for i in range(nvaldata):
    xvaldatanp[i][0]=random.uniform(0,1)
    xvaldatanp[i][1]=random.uniform(0,1)
    zvaldatanp[i]=(1.0/6.0)*((30.0+5.0*xvaldatanp[i][0]*np.sin(5.0*xvaldatanp[i][0]))*(4.0+np.exp(-5.0*xvaldatanp[i][1]))-100.0)


# Alamo is called with a number of keyword arguments
res=alamopy.doalamo(xdatanp,zdatanp,xval=xvaldatanp,zval=zvaldatanp,xlabels=('x1','x2'),xmin=(0,0),xmax=(1,1),monomialpower=(1,2,3,4),multi2power=(1,2,3),linfcns=1,expandoutput=True,simulator='limsim.py',savescratch=1,maxiter=5,tolmaxerror=7,tolesterror=2,sampler=1,showalm=False)


# Confidence intervals can only be calculated if expandoutput=True is used, or the data is provided explicitly as res = alamopy.almconfidence(res,xdata,zdata)
res = alamopy.almconfidence(res)

