##############################################################################
# Institute for the Design of Advanced Energy Systems Process Systems
# Engineering Framework (IDAES PSE Framework) Copyright (c) 2018, by the
# software owners: The Regents of the University of California, through
# Lawrence Berkeley National Laboratory,  National Technology & Engineering
# Solutions of Sandia, LLC, Carnegie Mellon University, West Virginia
# University Research Corporation, et al. All rights reserved.
# 
# Please see the files COPYRIGHT.txt and LICENSE.txt for full copyright and
# license information, respectively. Both files are also available online
# at the URL "https://github.com/IDAES/idaes".
##############################################################################
def model(infile):
    import ripe
    import sys, os
    import numpy as np
    from ripe import read, ratedet, writeripe

    data, deubg = ripe.data, ripe.debug
    # Read input data file
    read(infile)

    # If dynamic data has been specified, apply piecewise alamo
    if( 'time' in data['procdata']):
        [local_prof, local_r2] = doalamo(1,data['otherdata']['ntimedata'],range(data['otherdata']['npc']),data['otherdata']['slabels'])
        for l in list(['profiles','tlo','tup','etrack']):
            for pc in range(data['otherdata']['npc']):
                data['pwalamo'][l][pc]={}
                for s in data['otherdata']['slabels']:
                    data['pwalamo'][l][pc][s] = list()
        dopwalamo(local_prof, local_r2,1,data['otherdata']['ntimedata'],range(data['otherdata']['npc']),data['otherdata']['slabels'])

    # determines the experimental rates                                               
    ratedet()

    # This step is not automated at all right now, specifies reaction stoichiometry                                   
    netdet()
    # Check to see if ccon is specified
    try:
        ccon = data['otherdata']['ccon']
    except:
        ccon = -1

    # Initialize activity matrix
    if ('time' in data['procdata']):
        aterm=np.zeros([data['otherdata']['npc'],data['otherdata']['ntimedata'],data['rxnet']['nrx']])
    else:
        aterm=np.zeros([data['otherdata']['npc'],data['rxnet']['nrx']])

    # Call to write the RIPE MINLP
    writeripe(aterm,ccon)


def dopwalamo(profile, r2,xlo,xup,rpc,rspec):
    import numpy as np
    import ripe
    from sympy import symbols
    from sympy.parsing.sympy_parser import parse_expr
    import os

    data, debug = ripe.data, ripe.debug

    #python syntax of data indicies
    xlm=xlo-1
    xum=xup-1

    st = symbols('st')
    for pc in rpc:
        for s in rspec:
            amodel = parse_expr(profile[pc][s])
            if(float(r2[pc][s]) > data['pwalamo']['err']):
                data['pwalamo']['profiles'][pc][s].append(profile[pc][s])
                data['pwalamo']['tlo'][pc][s].append(xlo)
                data['pwalamo']['tup'][pc][s].append(xup)
                data['pwalamo']['etrack'][pc][s].append(r2)
            elif ((xup-xlo) > 2*data['pwalamo']['nmin']):
                cutoff = (1.0-float(data['pwalamo']['err']))*np.sum((data['concdata'][s][pc][xlm:xum] - np.average(data['concdata'][s][pc][xlm:xum]))**2)
                temp =0
                i = xlm
                while temp < 0.5 * cutoff:
                    temp = temp + (data['concdata'][s][pc][i] - amodel.subs(st,data['procdata']['time'][0][i]))**2
                    i=i+1
                if ((i-xlo) < data['pwalamo']['nmin']): i=xlo+data['pwalamo']['nmin']
                if ((xup-i) > data['pwalamo']['nmin']): i = xup-data['pwalamo']['nmin']
                [lp1, lr1] = doalamo(xlo,i,[pc],[s])
                dopwalamo(lp1,lr1,xlo,i,[pc],[s])
                [lp2, lr2] = doalamo(i,xup,[pc],[s])
                dopwalamo(lp2,lr2,i,xup,[pc],[s])
            else:
                data['pwalamo']['profiles'][pc][s].append(profile[pc][s])
                data['pwalamo']['tlo'][pc][s].append(xlo)
                data['pwalamo']['tup'][pc][s].append(xup)
                data['pwalamo']['etrack'][pc][s].append(r2[pc][s])


def ripeci(aterm,ccon):
    # Determine confidence intervals for kinetic parameters
    #Initialize variables                                  
    # cirxmat contains information for the derivatives of the slsected rxns
    # covar defines the covariance matrix                                  
    # phat is a 2xcc vector containing the kinetic paramteres                                                                                                                    
    import numpy as np
    import ripe

    data, deubg = ripe.data, ripe.debug

    if ('T' in data['otherdata']['pclabels']):
        covar = np.zeros([2*ccon,2*ccon])
        phat = np.zeros([4,ccon])
    else:
        covar = np.zeros([ccon,ccon])
        phat = np.zeros([4,ccon])
    res=open('res.put')
    res1=res.read()
    res.close()
    resdata=res1.split('\n')
    ci = np.zeros([2*ccon])
    errorobj = float(resdata[0])
    #Now read the chosen reactions
    j=0
    for i in range(1,len(resdata)-1):
        resvec=resdata[i].split(' ')
        if (int(resvec[2]) == 1):
            #This reaction is selected, populate appropriate arrays 
            phat[0,j] = float(resvec[3])
            if('T' in data['otherdata']['pclabels']):
                phat[1,j] = float(resvec[4])
            else:
                phat[1,j] = 0
            # reaction number and mechanism resp.
            phat[2,j] = int(resvec[0].split('.')[0])-1
            phat[3,j] = int(resvec[1].split('.')[0])
            j=j+1

    nr=0
    nd=0
    if(0):
#    if ('adata' in procdata):
        adsaux=np.zeros([data['otherdata']['npc'],ccon])
        for i in range(1+3*data['rxnet']['nrx'],len(resdata)-1):
            resvec=resdata[i].split(' ')
            if (int(resvec[0].split('.')[0])-1 in list(phat[2,:])):
                    adsaux[nd,nr]=float(resvec[1])
                    nr=nr+1
                    if (nr == ccon):
                        nd=nd+1
                        nr = 0

    # This section of old code is not currently used
    z=0
    z1=0
    j=0
    ndata = max(data['otherdata']['npc'],data['otherdata']['npc']*data['otherdata']['ntimedata'])
    aterm2=np.zeros([ndata,ccon])
    keq1=np.zeros(data['otherdata']['npc'])
    if('T' in data['otherdata']['pclabels']):
        keq2=np.zeros(data['otherdata']['npc'])
    if (1 == 0):
        t1=float(data['procdata']['T'][t])
        time=float(data['procdata']['time'][t][z1])
        keq1[t]=0.0005*(np.exp(500.0/(8.314*t1))/np.exp(1500.0/(8.314*t1)))*np.exp(30000.0/(8.314*t1))
        keq2[t]=0.001*np.exp(15000/(8.314*t1))
        for s in data['otherdata']['slabels']:
            # z indexed at this level
            for r in range(data['rxnet']['nrx']):
                # j indexed at this level
                # if reaction r is chosen & s is a product it will contribute to the activity term
                if (phat[2,j] == r):
                    # initialize the product to the stoichiometric coefficient 
                    aterm2[z,j] = float(data['rxnet'][r,s])
                    if (phat[3,j] == 1):
                        # The reaction is irreversible  
                        for s2 in data['otherdata']['slabels']:
                            if (data['rxnet'][phat[2,j],s2,1]>0):
                                aterm2[z,j] = aterm2[z,j] * float(data['concdata'][s2][t][z1])**float(data['rxnet'][phat[2,j],s2,1])
                    elif (phat[3,j] == 3):
                        for s2 in data['otherdata']['slabels']:
                            if (data['rxnet'][int(phat[2,j]),s2,1]>0):
                                aterm2[z,j] = aterm2[z,j] * float(data['concdata'][s2][t][z1])**float(data['rxnet'][phat[2,j],s2,1])
                    # apply adsorption effect
                    elif (phat[3,j] == 4):
                        aterm2[z,j] = aterm2[z,j] * float(data['concdata'][s2][t][z1])**float(data['rxnet'][phat[2,j],s2,1])
                    # reaction is reversible
                    elif (phat[3,j] == 2):
                        a1=1.0
                        b1=1.0
                        for s2 in data['otherdata']['slabels']:
                            # a is reactants b1 is products
                            if (data['rxnet'][phat[2,j],s2,1]>0):
                                a1 = a1*float(data['concdata'][s2][t])**float(data['rxnet'][phat[2,j],s2,1])
                            if (data['rxnet'][phat[2,j],s2,2]>0):
                                b1 = b1*float(data['concdata'][s2][t])**float(data['rxnet'][phat[2,j],s2,2])
                        if (j == 2):
                            aterm2[z,j]=aterm2[z,j]*float(a1 - b1/keq1[t])
                        elif (j == 5):
                            aterm2[z,j]=aterm2[z,j]*float(a1 - b1/keq2[t])
                        else:
                            sys.stdout.write('Problem in confidence interval calculation')
                    j=j+1
                    if (j == ccon):
                        j=0
                    z=z+1
                    z1=z1+1
                    if (z1 > data['otherdata']['timeseries']-1):
                        z1=0
    z=0
    for q in range(ccon):
        z = 0
        for i in range(data['otherdata']['npc']):
            if ('time' in data['procdata']):
                for j in range(data['otherdata']['ntimedata']):
                    aterm2[z,q]=aterm[i,j,int(phat[2,q])]
                    z=z+1
            else:
                #print earlier phat[3,q] was used
                aterm2[z,q] = aterm[i,int(phat[2,q])]
                z=z+1

    if('T' in data['otherdata']['pclabels']):
        sensmat=np.zeros([ndata,2*ccon])
    else:
        sensmat=np.zeros([ndata,ccon])
    if('T' in data['otherdata']['pclabels']):
        ovec=range(0,2*ccon-1,2)
    i=0
    for t in range(data['otherdata']['npc']):
        for q in range(data['otherdata']['ntimedata']):
#            for s in otherdata['spec']:
            for j in range(ccon):
                if('T' in data['otherdata']['pclabels']):
                    o=ovec[j]
                else:
                    o=j
                if(0):
#                if ('adata' in procdata):
                    sensmat[i,o]= adsaux[t,j]*aterm2[i,j]*np.exp(-1*phat[1,j]/(8.314*data['procdata']['T'][t]))
                    sensmat[i,o+1]= -1*adsaux[t,j]*aterm2[i,j]*phat[0,j]*np.exp(-1*phat[1,j]/(8.314*data['procdata']['T'][t]))*(1/(8.314*data['procdata']['T'][t]))
                elif ('clc' in data['otherdata']):
                    if('T' in data['otherdata']['pclabels']):
                        sensmat[i,o]= aterm2[i,j]*np.exp(-1*phat[1,j]/(8.314*data['procdata']['T'][t]))
                        sensmat[i,o+1]= -1*aterm2[i,j]*phat[0,j]*np.exp(-1*phat[1,j]/(8.314*data['procdata']['T'][t]))*(1/(8.314*data['procdata']['T'][t]))
                    else:
                        sensmat[i,o]= aterm2[i,j]
                else:
                    if( 'T' in otherdata['pclabels']):
                        sensmat[i,o]= aterm2[i,j]*np.exp(-1*phat[1,j]/(8.314*data['procdata']['T'][t]))
                        sensmat[i,o+1]=-1*aterm2[i,j]*phat[0,j]*np.exp(-1*phat[1,j]/(8.314*data['procdata']['T'][t]))*(1/(8.314*data['procdata']['T'][t]))
                    else:
                        sensmat[i,o]= aterm2[i,j]
            i=i+1
        if ('time' not in data['procdata']):
            for j in range(ccon):
                if('T' in data['otherdata']['pclabels']):
                    o=ovec[j]
                else:
                    o=j
                    sensmat[t,o]= aterm2[t,j]

    if('T' in data['otherdata']['pclabels']):
        sigma = errorobj / (ndata - 2*ccon)
    else:
        sigma = errorobj / (ndata - ccon)
    #Find condition number of A'A                                                           

    if ('T' in data['otherdata']['pclabels'] or ccon > 1):
        covar = np.linalg.inv(np.matmul(np.transpose(sensmat),sensmat))
    else:
        covar=np.zeros([1,1])
        for i in range(np.size(sensmat)):
            covar[0,0] = covar[0,0] + float(sensmat[i,0])**2
        covar[0,0] = 1 / float(covar[0,0])

    if('T' in data['otherdata']['pclabels']):
        for i in range(0,2*ccon):
            ci[i] = 1.725*np.sqrt( sigma * covar[i,i])
    else:
        for i in range(0,ccon):
            ci[i] = 1.725*np.sqrt( sigma * covar[i,i])

    for i in range(0,ccon):
#        if(otherdata['npc'] > 1):
        if('T' in data['otherdata']['pclabels']):
            o=ovec[i]
        else:
            o=i
        if ('clc' in data['otherdata']):
            sys.stdout.write(rspace['clcnames'][int(data['otherdata']['clc'][int(phat[3,i])-1].translate(None,'c'))-1]+'\n')
            sys.stdout.write(rspace['clcforms'][int(data['otherdata']['clc'][int(phat[3,i])-1].translate(None,'c'))-1]+'\n')
        sys.stdout.write('k('+str(phat[2,i])+','+str(phat[3,i])+') = '+str(phat[0,i])+' +/- '+str(ci[o])+'\n')
        if('T' in data['otherdata']['pclabels']):
            sys.stdout.write('Ea('+str(phat[2,i])+','+str(phat[3,i])+') = '+str(phat[1,i])+' +/- '+str(ci[o+1])+'\n')

    r2rep = 1.0 - errorobj / float(np.sum((data['ratedata'].values()-np.average(data['ratedata'].values()))**2))
    sys.stdout.write('R2 of the selected parameters is : ', r2rep)
    sys.stdout.write('average rate' ,np.average(data['ratedata'].values()))
    sys.stdout.write('error objective', errorobj)
    if ('time' in data['procdata']):
        sys.stdout.write('Number of pwAlamo models : ', len(data['pwalamo']['profiles'][0][data['otherdata']['slabels'][0]]))


def netdet():
    import ripe
    from ripe import data, debug
    # This section of code needs to be updated to read from input file / numpy array
    if ('clc' in data['otherdata']):
        data['rxnet'][0,'X']=1
        data['rxnet']['nrx']=1
        data['rxnet']['mechanisms']=len(data['otherdata']['clc'])
    elif (1 == 1):
        data['rxnet']['mechanisms'] = len(data['otherdata']['mechanisms'])
#        # true is 1 and 2
#        data['rxnet'][0,'a']=-1;   data['rxnet'][0,'b']=1;  data['rxnet'][0,'c']=0
#        data['rxnet'][1,'a']=0;    data['rxnet'][1,'b']=-1; data['rxnet'][1,'c']=1
#        data['rxnet'][2,'a']=-1;   data['rxnet'][2,'b']=-1;  data['rxnet'][2,'c']=2
#        data['rxnet'][3,'a']=1;   data['rxnet'][3,'b']=-2;  data['rxnet'][3,'c']=1
#        data['rxnet']['nrx']=4
#      
        data['rxnet'][0,'a']=-1;   data['rxnet'][0,'b']=-1;  data['rxnet'][0,'c']=1 ; data['rxnet'][0,'d']=0 ; data['rxnet'][0,'e']=0
        data['rxnet'][1,'a']=0;   data['rxnet'][1,'b']=-1;  data['rxnet'][1,'c']=-1 ; data['rxnet'][1,'d']=1 ; data['rxnet'][1,'e']=0
        data['rxnet'][2,'a']=-1;   data['rxnet'][2,'b']=0;  data['rxnet'][2,'c']=0 ; data['rxnet'][2,'d']=-1 ; data['rxnet'][2,'e']=1
        data['rxnet'][3,'a']=-1;   data['rxnet'][3,'b']=-2;  data['rxnet'][3,'c']=0 ; data['rxnet'][3,'d']=1 ; data['rxnet'][3,'e']=0
        data['rxnet'][4,'a']=-2;   data['rxnet'][4,'b']=-2;  data['rxnet'][4,'c']=0 ; data['rxnet'][4,'d']=0 ; data['rxnet'][4,'e']=1
        data['rxnet'][5,'a']=-1;   data['rxnet'][5,'b']=-1;  data['rxnet'][5,'c']=-1 ; data['rxnet'][5,'d']=0 ; data['rxnet'][5,'e']=1
        data['rxnet'][6,'a']=-2;   data['rxnet'][6,'b']=-1;  data['rxnet'][6,'c']=1 ; data['rxnet'][6,'d']=-1 ; data['rxnet'][6,'e']=1
        data['rxnet'][7,'a']=1;   data['rxnet'][7,'b']=0;  data['rxnet'][7,'c']=-1 ; data['rxnet'][7,'d']=-1 ; data['rxnet'][7,'e']=1
        data['rxnet'][8,'a']=-3;   data['rxnet'][8,'b']=-3;  data['rxnet'][8,'c']=1 ; data['rxnet'][8,'d']=0 ; data['rxnet'][8,'e']=1
        data['rxnet'][9,'a']=-3;   data['rxnet'][9,'b']=-4;  data['rxnet'][9,'c']=0 ; data['rxnet'][9,'d']=1 ; data['rxnet'][9,'e']=1
        data['rxnet'][10,'a']=-2;   data['rxnet'][10,'b']=-3;  data['rxnet'][10,'c']=1 ; data['rxnet'][10,'d']=1 ; data['rxnet'][10,'e']=0
        data['rxnet'][11,'a']=-4;   data['rxnet'][11,'b']=-5;  data['rxnet'][11,'c']=1 ; data['rxnet'][11,'d']=1 ; data['rxnet'][11,'e']=1
        data['rxnet']['nrx']=12
    else:
        # A + B > D
        data['rxnet'][0,'a']=-1;   data['rxnet'][0,'b']=-1;  data['rxnet'][0,'c']=0;  data['rxnet'][0,'d']=1;   data['rxnet'][0,'e']=0;  data['rxnet'][0,'f']=0;  data['rxnet'][0,'g']=0;   data['rxnet'][0,'h']=0;  data['rxnet'][0,'i']=0;   data['rxnet'][0,'j']=0;   data['rxnet'][0,'k']=0;
        # 2A + C > 2E
        data['rxnet'][1,'a']=-2;   data['rxnet'][1,'b']=0;  data['rxnet'][1,'c']=-1;   data['rxnet'][1,'d']=0;   data['rxnet'][1,'e']=0;  data['rxnet'][1,'f']=0;  data['rxnet'][1,'g']=0;   data['rxnet'][1,'h']=0;  data['rxnet'][1,'i']=0;   data['rxnet'][1,'j']=0;   data['rxnet'][1,'k']=0;
        # B + E > F
        data['rxnet'][2,'a']=0;   data['rxnet'][2,'b']=-1;   data['rxnet'][2,'c']=0;  data['rxnet'][2,'d']=0;   data['rxnet'][2,'e']=-1;  data['rxnet'][2,'f']=1;  data['rxnet'][2,'g']=0;   data['rxnet'][2,'h']=0;  data['rxnet'][2,'i']=0;   data['rxnet'][2,'j']=0;   data['rxnet'][2,'k']=0;
        # A + 2B > 2G
        data['rxnet'][3,'a']=-1;   data['rxnet'][3,'b']=-2;  data['rxnet'][3,'c']=0;   data['rxnet'][3,'d']=0;   data['rxnet'][3,'e']=0;  data['rxnet'][3,'f']=0;  data['rxnet'][3,'g']=2;   data['rxnet'][3,'h']=0;  data['rxnet'][3,'i']=0;   data['rxnet'][3,'j']=0;data['rxnet'][3,'k']=0
        # A + 2B + C > 2H
        data['rxnet'][4,'a']=-1;   data['rxnet'][4,'b']=-2;   data['rxnet'][4,'c']=-1;  data['rxnet'][4,'d']=0;   data['rxnet'][4,'e']=0;  data['rxnet'][4,'f']=0;  data['rxnet'][4,'g']=0;   data['rxnet'][4,'h']=0;  data['rxnet'][4,'i']=0;   data['rxnet'][4,'j']=0;data['rxnet'][4,'k']=0
        # 2E > J + 2G
        data['rxnet'][5,'a']=0;   data['rxnet'][5,'b']=0;   data['rxnet'][5,'c']=0;   data['rxnet'][5,'d']=0;   data['rxnet'][5,'e']=-2;  data['rxnet'][5,'f']=0;  data['rxnet'][5,'g']=2;   data['rxnet'][5,'h']=0;  data['rxnet'][5,'i']=0;  data['rxnet'][5,'j']=1;   data['rxnet'][5,'k']=0
        # 2B + 2J > K
        data['rxnet'][6,'a']=0;   data['rxnet'][6,'b']=-2;   data['rxnet'][6,'c']=0;   data['rxnet'][6,'d']=0;   data['rxnet'][6,'e']=0;  data['rxnet'][6,'f']=0;  data['rxnet'][6,'g']=0;   data['rxnet'][6,'h']=0;  data['rxnet'][6,'i']=0;   data['rxnet'][6,'j']=-2;   data['rxnet'][6,'k']=1
        # G + I > E
        data['rxnet'][7,'a']=0;   data['rxnet'][7,'b']=0;  data['rxnet'][7,'c']=0;  data['rxnet'][7,'d']=0;   data['rxnet'][7,'e']=1;  data['rxnet'][7,'f']=0;  data['rxnet'][7,'g']=-1;   data['rxnet'][7,'h']=0; data['rxnet'][7,'i']=-1;   data['rxnet'][7,'j']=0;   data['rxnet'][7,'k']=0
        # A+3B+E > F + 2G
        data['rxnet'][8,'a']=-1;   data['rxnet'][8,'b']=-3;  data['rxnet'][8,'c']=0;   data['rxnet'][8,'d']=0;   data['rxnet'][8,'e']=-1;  data['rxnet'][8,'f']=1;  data['rxnet'][8,'g']=2;   data['rxnet'][8,'h']=0;  data['rxnet'][8,'i']=0;   data['rxnet'][8,'j']=0;   data['rxnet'][8,'k']=0
        # A+2B+E > D+F
        data['rxnet'][9,'a']=-1;   data['rxnet'][9,'b']=-2;  data['rxnet'][9,'c']=0;  data['rxnet'][9,'d']=1;   data['rxnet'][9,'e']=-1;  data['rxnet'][9,'f']=1;  data['rxnet'][9,'g']=0;   data['rxnet'][9,'h']=0;  data['rxnet'][9,'i']=0;   data['rxnet'][9,'j']=0;   data['rxnet'][9,'k']=0
        # 3A+B+C > D+2E
        data['rxnet'][10,'a']=-3;  data['rxnet'][10,'b']=-1;  data['rxnet'][10,'c']=-1; data['rxnet'][10,'d']=1;  data['rxnet'][10,'e']=2; data['rxnet'][10,'f']=0; data['rxnet'][10,'g']=0; data['rxnet'][10,'h']=0; data['rxnet'][10,'i']=0; data['rxnet'][10,'j']=0; data['rxnet'][10,'k']=0
        # E+F > 2A+B+C
        data['rxnet'][11,'a']=2;  data['rxnet'][11,'b']=1; data['rxnet'][11,'c']=1;  data['rxnet'][11,'d']=0;  data['rxnet'][11,'e']=-1; data['rxnet'][11,'f']=-1; data['rxnet'][11,'g']=0;  data['rxnet'][11,'h']=0; data['rxnet'][11,'i']=0;  data['rxnet'][11,'j']=0;  data['rxnet'][11,'k']=0
        # 2A+3B+C > D + 2H
        data['rxnet'][12,'a']=-2;  data['rxnet'][12,'b']=-3;  data['rxnet'][12,'c']=-1; data['rxnet'][12,'d']=1;  data['rxnet'][12,'e']=0; data['rxnet'][12,'f']=0; data['rxnet'][12,'g']=0;  data['rxnet'][12,'h']=2; data['rxnet'][12,'i']=0;  data['rxnet'][12,'j']=0; data['rxnet'][12,'k']=0
        # C + 2F > 2B + 2H + 2I
        data['rxnet'][13,'a']=0; data['rxnet'][13,'b']=2;  data['rxnet'][13,'c']=-1;  data['rxnet'][13,'d']=0;  data['rxnet'][13,'e']=0; data['rxnet'][13,'f']=-2; data['rxnet'][13,'g']=0;  data['rxnet'][13,'h']=2; data['rxnet'][13,'i']=2;  data['rxnet'][13,'j']=0; data['rxnet'][13,'k']=0
        # 2J > C + 2I
        data['rxnet'][14,'a']=0;  data['rxnet'][14,'b']=0; data['rxnet'][14,'c']=1; data['rxnet'][14,'d']=0;  data['rxnet'][14,'e']=0; data['rxnet'][14,'f']=0; data['rxnet'][14,'g']=0;  data['rxnet'][14,'h']=0; data['rxnet'][14,'i']=2;  data['rxnet'][14,'j']=-2; data['rxnet'][14,'k']=0
        # 2B+I+J>K
        data['rxnet'][15,'a']=0;  data['rxnet'][15,'b']=-2;  data['rxnet'][15,'c']=0; data['rxnet'][15,'d']=0;  data['rxnet'][15,'e']=0; data['rxnet'][15,'f']=0; data['rxnet'][15,'g']=0;  data['rxnet'][15,'h']=0; data['rxnet'][15,'i']=-1;  data['rxnet'][15,'j']=-1; data['rxnet'][15,'k']=1
        # G+2J>C+E+I
        data['rxnet'][16,'a']=0;  data['rxnet'][16,'b']=0; data['rxnet'][16,'c']=1;  data['rxnet'][16,'d']=0; data['rxnet'][16,'e']=1; data['rxnet'][16,'f']=0; data['rxnet'][16,'g']=-1;  data['rxnet'][16,'h']=0; data['rxnet'][16,'i']=1;  data['rxnet'][16,'j']=-2; data['rxnet'][16,'k']=0
        # D+K>A+3B+I+J
        data['rxnet'][17,'a']=1;  data['rxnet'][17,'b']=3; data['rxnet'][17,'c']=0;  data['rxnet'][17,'d']=-1; data['rxnet'][17,'e']=0; data['rxnet'][17,'f']=0; data['rxnet'][17,'g']=0;  data['rxnet'][17,'h']=0; data['rxnet'][17,'i']=1;  data['rxnet'][17,'j']=1; data['rxnet'][17,'k']=-1
        # F+K>3B+E+I+J
        data['rxnet'][18,'a']=0;  data['rxnet'][18,'b']=3; data['rxnet'][18,'c']=0;  data['rxnet'][18,'d']=0; data['rxnet'][18,'e']=1; data['rxnet'][18,'f']=-1; data['rxnet'][18,'g']=0;  data['rxnet'][18,'h']=0; data['rxnet'][18,'i']=1;  data['rxnet'][18,'j']=1; data['rxnet'][18,'k']=-1
        # C+G>2B+I
        data['rxnet'][19,'a']=0;  data['rxnet'][19,'b']=2; data['rxnet'][19,'c']=-1;  data['rxnet'][19,'d']=0; data['rxnet'][19,'e']=0; data['rxnet'][19,'f']=0; data['rxnet'][19,'g']=-1;  data['rxnet'][19,'h']=0; data['rxnet'][19,'i']=1;  data['rxnet'][19,'j']=0; data['rxnet'][19,'k']=0

        data['rxnet']['mechanisms']=4
        data['rxnet']['ma']=list([0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19])
        data['rxnet']['rev']=list([5])
        data['rxnet']['homcat']=list([0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19])
        data['rxnet']['ads'] = list([0])
        data['rxnet']['nrx']=20

    # This section has recieved some recent love and may need more work to generalize to older files
    gset=1
    for i in range(data['rxnet']['nrx']):
        data['gprxindex'][i]={}
        for m in data['otherdata']['mechanisms']:
            if (m == 'clc'):
                s='X'
                for cm in data['otherdata']['clc']:
                    data['gprxindex'][i][cm]=gset
                    data['rxnet'][i,s,cm]={}
                    if data['rxnet'][i,s] < 0:
                        data['rxnet'][i,s,cm]=abs(data['rxnet'][i,s])
                    elif data['rxnet'][i,s] > 0:
                        data['rxnet'][i,s,cm]=data['rxnet'][i,s]
                    else:
                        data['rxnet'][i,s,cm]=0
                    gset=gset+1
            else:
                data['gprxindex'][i][m]=gset
                gset=gset+1
                for s in otherdata['spec']:
                    data['rxnet'][i,s,m]={}
                    if data['rxnet'][i,s] < 0:
                        data['rxnet'][i,s,m]=abs(data['rxnet'][i,s])
                    elif data['rxnet'][i,s] > 0:
                        data['rxnet'][i,s,m]=data['rxnet'][i,s]
                    else:
                        data['rxnet'][i,s,m]=0



def ratedet():
    import ripe
    import sympy
    from sympy import symbols, diff
    import numpy as np

    data, debug = ripe.data, ripe.debug

    bignum = debug['bignum']
    mthd='deriv'
    if ( 'time' in data['procdata'] ):
        for s in data['otherdata']['slabels']:
            data['ratedata'][s]=np.zeros([data['otherdata']['timeseries'],data['otherdata']['ntimedata']])
    else:
        for s in data['otherdata']['slabels']:
            data['ratedata'][s]=np.zeros([data['otherdata']['npc']])
            if 'npcval' in data['otherdata']:
                data['ratevaldata'][s]=np.zeros([data['otherdata']['npcval']])
    st=symbols('st')
    # Noise can be added at input-level
    if ( 'noise' in data['otherdata'] ):
        sc = 0
        mag = {}
        for s in data['otherdata']['slabels']:
            try:
                mag[s]=float(data['otherdata']['noise'][sc])
            except:
                mag[s] = 0.0
            sc=sc+1
    if ('time' in data['procdata']):
        for s in data['otherdata']['slabels']:
            for i in range(data['otherdata']['npc']):
                mthd = 'deriv'
                #creat tracker index for pwalamo
                pwt = 0
                for j in range(data['otherdata']['ntimedata']):
                    if mthd == 'deriv':
                        if debug['true']['use'] == False:
                            profile = data['pwalamo']['profiles'][i][s][pwt]
                            data['ratedata'][s][i][j] = diff(profile,st).subs(st,data['procdata']['time'][i][j])
                            if j == data['pwalamo']['tup'][i][s][pwt]:
                                pwt=pwt+1
                        else:
                            data['ratedata'][s][i][j] = (debug['true']['k']*exp(-debug['true']['e']/(8.314*data['procdata']['T'][i]))*exp(-debug['true']['k']*exp(-debug['true']['e']/(8.314*data['procdata']['T'][i]))*st)).subs(st,data['procdata']['time'][i][j])
                    if (mthd == 'fd'):
                        if (j==0):
                            data['ratedata'][s][i][j]=float(float(data['procdata']['c0data'][s])-float(data['concdata'][s][i][j]))/float(data['procdata']['time'][i][j])
                        else:
                            data['ratedata'][s][i][j]=float(float(data['concdata'][s][i][j])-float(data['concdata'][s][i][j-1]))/float(data['procdata']['time'][i][j]-data['procdata']['time'][i][j-1])
                    if 'noise' in data['otherdata']:
                        data['ratedata'][s][i][j]=data['ratedata'][s][i][j]+np.random.normal(0,mag[s])
                    if data['ratedata'][s][i][j] != data['ratedata'][s][i][j] : data['ratedata'][s][i][j]=0
                    if data['ratedata'][s][i][j] < -1 * debug['bignum'] : data['ratedata'][s][i][j] = 0
                    if data['ratedata'][s][i][j] > debug['bignum'] : data['ratedata'][s][i][j] = 0
    else:
        for s in data['otherdata']['slabels']:
            for j in range(data['otherdata']['npc']):
                if ('x0' in data['otherdata']['pclabels']):
                    c0temp = data['procdata']['c0data'][s][j]
                else:
                    c0temp = data['procdata']['c0data'][s]
                if ( 'noise' in data['otherdata'] ):
                    data['ratedata'][s][j] = (float(data['concdata'][s][j])+np.random.normal(0,mag[s])-float(c0temp))/(float(data['otherdata']['volume']))
                else:
                    data['ratedata'][s][j] = (float(data['concdata'][s][j])-float(c0temp))/(float(data['otherdata']['volume']))
                if data['ratedata'][s][j] != data['ratedata'][s][j]: data['ratedata'][s][j]=0
                if data['ratedata'][s][j] < -1 * bignum: data['ratedata'][s][j] = 0
                if data['ratedata'][s][j] > bignum: data['ratedata'][s][j] = 0
            if ('npcval' in otherdata):
                for j in range(data['otherdata']['npcval']):
                    if ('noise' in otherdata):
                        data['ratedata'][s][j] = (float(data['concvaldata'][s][j])+np.random.normal(0,mag[s])-float(data['procdata']['c0data'][s]))/(float(data['otherdata']['volume']))
                    else:
                        data['ratedata'][s][j] = (float(data['concvaldata'][s][j])-float(data['procdata']['c0data'][s]))/(float(data['otherdata']['volume']))
                    if data['ratevaldata'][s][j] != ratedata[s][j] : data['ratedata'][s][j]=0
                    if data['ratevaldata'][s][j] < -1 * bignum : data['ratedata'][s][j] = 0
                    if data['ratevaldata'][s][j] > bignum : data['ratedata'][s][j] = 0

#def doalamo(procdata,otherdata,profiles,debug):
def doalamo(xlo,xup,rpc,rspec):
    import ripe
    import sympy
    from sympy import symbols
    import os
    from alamopy import deletefile, copyfile

    data, debug, rspace = ripe.data, ripe.debug, ripe.rspace
    
    #Convert data to python index
    xlo=xlo-1
    xup=xup-1

    r2ret = {}
    profiles = {}
    st=symbols('st')
    for p in rpc:
        r2ret[p] = {}
        profiles[p]={}
        with open('temp.alm', 'w') as a:
            a.write('linfcns 1\n')
#            a.write('enumall 0\n')
#            a.write('enum1 0\n')
#            a.write('builder 0\n')
            a.write('funform 1\n')
            a.write('trace 1\n')
            a.write('tracefname temptrace.trc\n')
            a.write('solvemip 1\n')
            a.write('ninputs 1\n')
            a.write('noutputs '+str(len(data['otherdata']['slabels']))+'\n')
            a.write('xlabels st\n')
            a.write('modeler 1\n')
            if debug['true']['use'] == False:
#                a.write('monomialpower 1 1.5 2 2.5 3 3.5 4 4.5 5 5.5\n')
#                a.write('monomialpower 0.2 0.5 0.7 1 1.2 1.5 1.7 2 2.5 3\n')
                a.write('monomialpower 0.5 1 1.5 2 \n')
                a.write('logfcns 1\n')
                a.write('expfcns 1\n')
                a.write('constant 0\n')
            else:
                a.write('ncustombas 1\n')
                this=debug['true']['k'] * exp(-debug['true']['e']/(8.314*procdata['T'][p]))
                a.write('begin_custombas\n')
                a.write(str(this)+'*exp(-'+str(this)+'*st)\n')
                a.write('end_custombas\n')
            a.write('xmin 0\n')
            a.write('xmax '+str(data['procdata']['time'][0][-1])+'\n')
            a.write('cvxbic 1\n')
            a.write('initialpoints '+str(len(range(xlo,xup)))+'\n')
            a.write('ndata '+str(len(range(xlo,xup)))+'\n')
            a.write('begin_data\n')
            for i in range(xlo,xup):#range(otherdata['ntimedata']):
                tstr=str(data['procdata']['time'][0][i])+' '
                for j in rspec:
                    tstr=tstr+str(data['concdata'][j][p][i])+' '
                a.write(tstr+'\n')
            a.write('end_data\n')
        os.system("alamo temp.alm > tmpscratch")

    f = open('temptrace.trc')
    lf=f.read()
    f.close()
    lf2=lf.split('\n')
    z=0
    for i in rpc:
        for j in rspec:
            if ('#filename' in lf2[z]):
                z=z+1
            profiles[i][j]=lf2[z].split(',')[-1].split(' = ')[-1]
            r2ret[i][j]=lf2[z].split(',')[18]
            z=z+1

    copyfile('sv.alm','temp.alm')
    if debug['savescratch'] == False:
        deletefile('temp.alm','temptrace.trc','temp.lst')

    return [profiles, r2ret]
