##############################################################################
# Institute for the Design of Advanced Energy Systems Process Systems
# Engineering Framework (IDAES PSE Framework) Copyright (c) 2018, by the
# software owners: The Regents of the University of California, through
# Lawrence Berkeley National Laboratory,  National Technology & Engineering
# Solutions of Sandia, LLC, Carnegie Mellon University, West Virginia
# University Research Corporation, et al. All rights reserved.
# 
# Please see the files COPYRIGHT.txt and LICENSE.txt for full copyright and
# license information, respectively. Both files are also available online
# at the URL "https://github.com/IDAES/idaes".
##############################################################################
"""
Set the alamo and gams paths here.
"""
import collections as col

#r space is a dictionary containing input options 
#olabels is a dictionary containing internal working options
rspace = {}
rspace['inopt']=list(['simulator','slabels','pclabels','npc','npcval','timeseries',
                      'ntimedata','begin_concdata','end_concdata','begin_c0data',
                      'end_c0data','begin_tdata','end_tdata','begin_vdata','end_vdata',
                      'mechanisms','c0vary','volume','adsspec','adslabel','begin_ads',
                      'end_ads','ccon','eobj','revlabel','revspec','begin_rev','end_rev',
                      'noise','clc','begin_valdata','end_valdata','nvaldata','savescratch'])

olabels = list(['simulator','species','processconditions','time','concentrations','initialconc',
                'conversion','mechanisms','temperature','volume','adsorptioneq','cardinality',
                'reverseeq','errorobj','revspecies','simnoise','clc','valdatafraction'])

# Input-specific options and mechanisms
# Mechanisms used in CLC models are enumerated as strings are parsed later
# dlist old clcmlist
dlist = ['c1','c2','c3','c4','c5','c6','c7','c8','c9','c10','c11','c12','c13','c14','c15','c16','c17','c18','c19','c20','c21']
rspace['mechanisms']=list(['iT','ma','rev','homcat','clc','ads']+dlist)
rspace['clc']=list(dlist)
rspace['clcnames']=list(['Random Nucleation','Power law order 2/3','Power law order 2',
                         'Power law order 4','Power law order 3','Power law order 1',
                         'Avrami order 2','Avrami order 3','Avrami order 0.5','Avrami order 1.5',
                         'Avrami order 4','Prout Tompkins','Jander 3d diff','Anti Jander 3d diff',
                         'Valensi','Parabolic','Ginstling-Brountstein 3d diff',
                         'zhuralev lesohkin tempelman','Grain Model'])
rspace['clcforms']=list(['1-X','(n=2/3) nX^(n-1/n)','(n=2) nX^(n-1/n)',
                         '(n=4) nX^(n-1/n)','(n=3) nX^(n-1/n)','(n=1) nX^(n-1/n)',
                         '(n=2) n(1-X)(-ln(1-X))^(n-1/n)','(n=3) n(1-X)(-ln(1-X))^(n-1/n)',
                         '(n=0.5) n(1-X)(-ln(1-X))^(n-1/n)','(n=1.5) n(1-X)(-ln(1-X))^(n-1/n)',
                         '(n=4) n(1-X)(-ln(1-X))^(n-1/n)','X(1-X)',
                         '3(1-X)^(1/3)(1/(1+X)^((-1/3)-1))','3/2(1-X)^(2/3)(1/(1+X)^((-1/3)-1))',
                         '1/(-ln(1-X))','1/(2X)','(3/2)(1-X)^(4/3)((1-X)^((-1/3)-1))^-1',
                         '(3/2)((1-X)^((-1/3)-1))^-1','(1-X)^(2/3)'])
rspace['pclabels']=list(['T','v','x0'])

#Initialize additional dictionaries, and initialize options that will be passed between subroutines
data = {}
dlist = list(['procdata','procvaldata','concdata','concvaldata','ratedata','ratevaldata','c0data','rxnet','gprxindex','profiles','otherdata','pwalamo'])
for item in dlist:
    data[item] = {}
dlist = list(['profiles','tlo','tup','etrack','err','nmin'])
for item in dlist:
    data['pwalamo'][item] = {}
# Specify options pertaining to applicaiton of pw-alamo
data['pwalamo']['err']  = 0.99
data['pwalamo']['nmin'] = 3
data['otherdata']['eobj'] = 2

debug = {}
debug['savescratch'] = False
debug['true'] = {}
debug['bignum'] = 10**20
debug['true']['use']=False
debug['true']['fix']=False
